@@ -25,78 +25,102 @@
 #
 # ===============================================================================
 
-
-make_overlay_raster = function(las, res, start = c(0,0), subcircle = 0)
+#' RasterLayer tools
+#'
+#' Internal function: a set of tools related to \code{Raster*}
+#' \itemize{
+#' \item \code{rOverlay}: Generates an (empty) \code{RasterLayer} that encompasse a point cloud
+#' \item \code{rMergeList}: Merge a list of \code{Raster*} in a single \code{Raster*} preserving the
+#' layer names
+#' \item \code{rBuildVRT}: Create a Virtual Raster Mosaic from a set of Raster file on disk.
+#' }
+#'
+#' @param las An object of class \code{LAS}
+#' @param res numeric. resolution of the raster
+#' @param start vector of x and y coordinates for the reference raster. Default is (0,0) meaning that
+#' the grid aligns on (0,0).
+#' @param buffer numeric. Extend the bounding box of the LAS object to generate a RasterLayer larger
+#' than the point cloud
+#' @param raster_list list. List of \code{Raster*} objects
+#' @param file_list list. List of path to \code{Raster*} file
+#' @param vrt charcter. Name of the VRT that must be written
+#'
+#' @keywords internal
+rOverlay = function(las, res, start = c(0,0), buffer = 0)
 {
   if (is(res, "RasterLayer"))
   {
-    resolution = raster::res(res)
-    if (resolution[1] !=  resolution[2]) stop("Rasters with different x y resolutions are not supported")
+    resolution <- raster::res(res)
+    if (resolution[1] !=  resolution[2])
+      stop("Rasters with different x y resolutions are not supported", call. = FALSE)
     return(res)
   }
 
-  bbox      <- raster::extent(las) + 2 * subcircle
+  bbox      <- raster::extent(las) + 2 * buffer
   bbox@xmin <- round_any(bbox@xmin - 0.5 * res - start[1], res) + start[1]
   bbox@xmax <- round_any(bbox@xmax - 0.5 * res - start[1], res) + res + start[1]
   bbox@ymin <- round_any(bbox@ymin - 0.5 * res - start[2], res) + start[2]
   bbox@ymax <- round_any(bbox@ymax - 0.5 * res - start[2], res) + res + start[2]
   layout    <- suppressWarnings(raster::raster(bbox, res = res, crs = las@proj4string))
+  raster::projection(layout) <- raster::projection(las)
   return(layout)
 }
 
-merge_rasters = function(output)
+#' @rdname rOverlay
+rMergeList = function(raster_list)
 {
-  if (length(output) == 1)
-    return(output)
+  if (length(raster_list) == 1)
+    return(raster_list)
 
-  names    <- names(output[[1]])
-  factor   <- output[[1]]@data@isfactor
-  issingle <- sapply(output, function(x) { raster::nrow(x) == 1 & raster::ncol(x) == 1 })
+  names    <- names(raster_list[[1]])
+  factor   <- raster_list[[1]]@data@isfactor
+  issingle <- sapply(raster_list, function(x) { raster::nrow(x) == 1 & raster::ncol(x) == 1 })
   single   <- list()
 
   if (any(issingle))
   {
-    single <- output[issingle]
-    output <- output[!issingle]
+    single <- raster_list[issingle]
+    raster_list <- raster_list[!issingle]
   }
 
-  output <- do.call(raster::merge, output)
-  names(output) <- names
+  raster <- do.call(raster::merge, raster_list)
+  names(raster) <- names
 
-  if (is(output, "RasterBrick") & raster::inMemory(output))
-    colnames(output@data@values) <- names
+  if (is(raster, "RasterBrick") && raster::inMemory(raster))
+    colnames(raster@data@values) <- names
 
   for (pixel in single)
   {
     pix = raster::rasterToPoints(pixel, spatial = TRUE)
-    output[pix] <- as.matrix(pix@data)
+    raster[pix] <- as.matrix(pix@data)
   }
 
-  return(output)
+  return(raster)
 }
 
-build_vrt = function(output, vrt)
+#' @rdname rOverlay
+rBuildVRT = function(file_list, vrt)
 {
   if (!options("lidR.buildVRT")[[1]])
-    return(unlist(output))
+    return(unlist(file_list))
 
   if (!requireNamespace("gdalUtils", quietly = TRUE))
   {
     message("'gdalUtils' package is needed to build a virtual raster mosaic. Returns the list of written files instead.")
-    return(unlist(output))
+    return(unlist(file_list))
   }
 
-  output <- unlist(output)
-  folder <- dirname(output[1])
-  file   <- paste0("/", vrt, ".vrt")
-  vrt    <- paste0(folder, file)
-  gdalUtils::gdalbuildvrt(output, vrt)
-  output <- raster::stack(vrt)
+  file_list <- unlist(file_list)
+  folder    <- dirname(file_list[1])
+  file      <- paste0("/", vrt, ".vrt")
+  vrt       <- paste0(folder, file)
+  gdalUtils::gdalbuildvrt(file_list, vrt)
+  file_list <- raster::stack(vrt)
 
-  if (dim(output)[3] == 1)
-    return(output[[1]])
+  if (dim(file_list)[3] == 1)
+    return(file_list[[1]])
   else
-    return(output)
+    return(file_list)
 }
 
 match_chm_and_seeds = function(chm, seeds, field)
@@ -105,7 +129,7 @@ match_chm_and_seeds = function(chm, seeds, field)
   assert_is_all_of(seeds, "SpatialPointsDataFrame")
   stopif_forbidden_name(field)
 
-  if(is.null(raster::intersect(raster::extent(chm), raster::extent(seeds))))
+  if (is.null(raster::intersect(raster::extent(chm), raster::extent(seeds))))
     stop("No seed matches with the canopy height model" )
 
   if (field %in% names(seeds@data))
