@@ -1,5 +1,10 @@
 #include "QuadTree.h"
 
+QuadTree::QuadTree()
+{
+  init();
+}
+
 QuadTree::QuadTree(const double xcenter, const double ycenter, const double range)
 {
   use3D = false;
@@ -11,6 +16,7 @@ QuadTree::QuadTree(const BoundingBox boundary, const QuadTree* parent)
 {
   init();
   this->boundary = boundary;
+  this->MAX_DEPTH = parent->MAX_DEPTH;
   this->depth = parent->depth + 1;
   this->Z = parent->Z;
   this->use3D = parent->use3D;
@@ -38,6 +44,17 @@ QuadTree::QuadTree(Rcpp::S4 las)
   init(x,y,z);
 }
 
+QuadTree::QuadTree(Rcpp::S4 las, std::vector<bool>& f)
+
+{
+  Rcpp::DataFrame data = Rcpp::as<Rcpp::DataFrame>(las.slot("data"));
+  Rcpp::NumericVector x = data["X"];
+  Rcpp::NumericVector y = data["Y"];
+  Rcpp::NumericVector z = data["Z"];
+  use3D = true;
+  init(x,y,z,f);
+}
+
 QuadTree::~QuadTree()
 {
   delete NE;
@@ -155,6 +172,34 @@ void QuadTree::init()
 }
 
 void QuadTree::init(Rcpp::NumericVector x, Rcpp::NumericVector y)
+{
+  if (x.size() != y.size())
+    throw(std::runtime_error("Internal error in QuadTree. x and y have different sizes."));
+
+  init();
+  unsigned int n = x.size();
+  double xmin = Rcpp::min(x);
+  double ymin = Rcpp::min(y);
+  double xmax = Rcpp::max(x);
+  double ymax = Rcpp::max(y);
+  double xrange = xmax - xmin;
+  double yrange = ymax - ymin;
+  double range = xrange > yrange ? xrange/2 : yrange/2;
+  boundary = BoundingBox(Point((xmin+xmax)/2, (ymin+ymax)/2), Point(range+0.001, range+0.001));
+
+  int computed_depth = std::floor(std::log(n)/std::log(4));
+  computed_depth = (computed_depth >= 1) ? computed_depth : 1;
+  computed_depth = (computed_depth >= 8) ? 8 : computed_depth;
+  MAX_DEPTH = computed_depth;
+
+  for(int i = 0 ; i < x.size() ; i++)
+  {
+    Point p(x[i], y[i], i);
+    insert(p);
+  }
+}
+
+void QuadTree::init(Rcpp::NumericVector x, Rcpp::NumericVector y, std::vector<bool>& f)
 {
   if (x.size() != y.size())
     throw(std::runtime_error("Internal error in QuadTree. x and y have different sizes."));
@@ -171,8 +216,11 @@ void QuadTree::init(Rcpp::NumericVector x, Rcpp::NumericVector y)
 
   for(int i = 0 ; i < x.size() ; i++)
   {
-    Point p(x[i], y[i], i);
-    insert(p);
+    if (f[i])
+    {
+      Point p(x[i], y[i], i);
+      insert(p);
+    }
   }
 }
 
@@ -186,6 +234,16 @@ void QuadTree::init(Rcpp::NumericVector x, Rcpp::NumericVector y, Rcpp::NumericV
   init(x,y);
 }
 
+void QuadTree::init(Rcpp::NumericVector x, Rcpp::NumericVector y, Rcpp::NumericVector z, std::vector<bool>& f)
+{
+  if (x.size() != z.size())
+    throw(std::runtime_error("Internal error in QuadTree. x and z have different sizes."));
+
+  Z = z;
+  init();
+  init(x,y,f);
+}
+
 BoundingBox QuadTree::bbox()
 {
   return this->boundary;
