@@ -9,9 +9,11 @@
 class QuadTree
 {
 	public:
+	  QuadTree();
 	  QuadTree(Rcpp::NumericVector x, Rcpp::NumericVector y);
 	  QuadTree(Rcpp::NumericVector x, Rcpp::NumericVector y, Rcpp::NumericVector z);
 	  QuadTree(Rcpp::S4 las);
+	  QuadTree(Rcpp::S4 las, std::vector<bool>&);
 	  ~QuadTree();
 		bool insert(const Point&);
 		template<typename T> void lookup(T& shape, std::vector<Point*>&); // solve issue with const correctness
@@ -41,7 +43,9 @@ private:
   void subdivide();
   void init();
   void init(Rcpp::NumericVector x, Rcpp::NumericVector y);
+  void init(Rcpp::NumericVector x, Rcpp::NumericVector y, std::vector<bool>&);
   void init(Rcpp::NumericVector x, Rcpp::NumericVector y, Rcpp::NumericVector z);
+  void init(Rcpp::NumericVector x, Rcpp::NumericVector y, Rcpp::NumericVector z,std::vector<bool>&);
 };
 
 template<typename T> void QuadTree::lookup(T& shape, std::vector<Point*>& res)
